@@ -46,7 +46,14 @@ class Test(object):
         expected = cairo.ImageSurface.create_from_png(path)
 
         if result.get_data() != expected.get_data():
-            return 'Image differs from expectation: %s\n' % self.src
+            error = 'Image differs from expectation: %s\n' % self.src
+            results_dir = os.environ.get('CAIRO_TEST_RESULTS')
+            if results_dir:
+                result_path = os.path.join(results_dir,
+                                           self.src.replace('/', '-'))
+                result.write_to_png(result_path)
+                error += '(see %s)\n' % result_path
+            return error
 
 
 class Result(object):
@@ -78,7 +85,8 @@ class Manuel(manuel.Manuel):
     def parse(self, document):
         for region in document.find_regions(self.Test.finder):
             if not region.parsed:
-                document.replace_region(region, self.Test(document, region))
+                document.claim_region(region)
+                region.parsed = self.Test(document, region)
 
     def evaluate(self, region, document, globs):
         if region.evaluated:
@@ -98,12 +106,12 @@ def DocFileSuite(path, module_relative=True, package=None,
                  setUp=None, tearDown=None, globs=None,
                  optionflags=0, checker=None, encoding=None,
                  manuel_object=None):
-    m = manuel.doctest.Manuel(optionflags=optionflags, checker=checker)
-    m.extend(manuel.footnote.Manuel())
-    m.extend(manuel.codeblock.Manuel())
-    m.extend(Manuel(Test, Result))
+    m = (manuel.doctest.Manuel(optionflags=optionflags, checker=checker)
+         + manuel.footnote.Manuel()
+         + manuel.codeblock.Manuel()
+         + Manuel(Test, Result))
     if manuel_object:
-        m.extend(manuel_object)
+        m += manuel_object
 
     if module_relative:
         calling_module = doctest._normalize_module(package)
